/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2011 Adrian Sai-wah Tam
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Adrian Sai-wah Tam <adrian.sw.tam@gmail.com>
 */

#ifndef TCP_OPTION_SACK_H
#define TCP_OPTION_SACK_H

#include "ns3/tcp-option.h"
#include "ns3/sequence-number.h"

namespace ns3 {

/**
 * Defines the TCP option of kind 5 (Selective Acknowledgment option) as in \RFC{2018}
 */

typedef std::pair<SequenceNumber32, SequenceNumber32> SackBlock;
typedef std::list<SackBlock> SackList;

class TcpOptionSack : public TcpOption
{
public:
  TcpOptionSack ();
  virtual ~TcpOptionSack ();

  /**
   * \brief Get the type ID.
   * \return the object TypeId
   */
  static TypeId GetTypeId (void);
  virtual TypeId GetInstanceTypeId (void) const;

  virtual void Print (std::ostream &os) const;
  virtual void Serialize (Buffer::Iterator start) const;
  virtual uint32_t Deserialize (Buffer::Iterator start);

  virtual uint8_t GetKind (void) const;
  virtual uint32_t GetSerializedSize (void) const;

  /**
   * \brief Add a SACK block
   * \param s the SACK block
   */
  void AddSackBlock (SackBlock s);
  /**
   * \brief Count the total number of SACK blocks
   * \return the total number of SACK blocks
   */
  uint32_t CountSackBlock (void) const;
  /**
   * \brief Clear the SACK list
   */
  void ClearSackList (void);
  /**
   * \brief Get the SACK list
   * \return the SACK list
   */
  SackList GetSackList (void) const;

protected:
  SackList m_sackList;       //!< the list of SACK blocks
};

} // namespace ns3

#endif /* TCP_OPTION_SACK */
